@@ -555,7 +555,7 @@ avatar_chooser_maybe_convert_and_scale (EmpathyAvatarChooser *chooser,
 
 	/* Takes ownership of new_mime_type and converted_image_data */
 	avatar = empathy_avatar_new (converted_image_data,
-		converted_image_size, new_mime_type, NULL);
+		converted_image_size, new_mime_type, NULL, NULL);
 
 	return avatar;
 }
@@ -598,7 +598,7 @@ avatar_chooser_set_image_from_data (EmpathyAvatarChooser *chooser,
 	}
 
 	/* avatar takes ownership of data and mime_type */
-	avatar = empathy_avatar_new (data, size, mime_type, NULL);
+	avatar = empathy_avatar_new (data, size, mime_type, NULL, NULL);
 
 	avatar_chooser_set_image (chooser, avatar, pixbuf, set_locally);
 }
