@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,13 @@
 G_BEGIN_DECLS
 
 void empathy_subscription_dialog_show        (EmpathyContact *contact,
-				              GtkWindow     *parent);
+				              GtkWindow      *parent);
 void empathy_contact_information_dialog_show (EmpathyContact *contact,
-					      GtkWindow     *parent,
-					      gboolean       edit,
-					      gboolean       is_user);
-void empathy_new_contact_dialog_show         (GtkWindow     *parent);
+					      GtkWindow      *parent);
+void empathy_contact_edit_dialog_show        (EmpathyContact *contact,
+					      GtkWindow      *parent);
+void empathy_contact_personal_dialog_show    (GtkWindow      *parent);
+void empathy_new_contact_dialog_show         (GtkWindow      *parent);
 
 G_END_DECLS
 
