@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  */
@@ -29,7 +29,6 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
@@ -144,7 +143,7 @@ empathy_log_window_show (McAccount   *account,
 	EmpathyAccountChooser   *account_chooser;
 	GList                  *accounts;
 	gint                    account_num;
-	GladeXML               *glade;
+	GtkBuilder             *gui;
 	gchar                  *filename;
 
 	if (window) {
@@ -160,13 +159,11 @@ empathy_log_window_show (McAccount   *account,
 	}
 
 	window = g_new0 (EmpathyLogWindow, 1);
-	window->log_manager = empathy_log_manager_new ();
+	window->log_manager = empathy_log_manager_dup_singleton ();
 
-	filename = empathy_file_lookup ("empathy-log-window.glade",
+	filename = empathy_file_lookup ("empathy-log-window.ui",
 					"libempathy-gtk");
-	glade = empathy_glade_get_file (filename,
-				       "log_window",
-				       NULL,
+	gui = empathy_builder_get_file (filename,
 				       "log_window", &window->window,
 				       "notebook", &window->notebook,
 				       "entry_find", &window->entry_find,
@@ -183,8 +180,7 @@ empathy_log_window_show (McAccount   *account,
 				       NULL);
 	g_free (filename);
 
-	empathy_glade_connect (glade,
-			      window,
+	empathy_builder_connect (gui, window,
 			      "log_window", "destroy", log_window_destroy_cb,
 			      "entry_find", "changed", log_window_entry_find_changed_cb,
 			      "button_previous", "clicked", log_window_button_previous_clicked_cb,
@@ -194,7 +190,7 @@ empathy_log_window_show (McAccount   *account,
 			      "entry_chats", "activate", log_window_entry_chats_activate_cb,
 			      NULL);
 
-	g_object_unref (glade);
+	g_object_unref (gui);
 
 	g_object_add_weak_pointer (G_OBJECT (window->window),
 				   (gpointer) &window);
@@ -290,9 +286,9 @@ log_window_entry_find_changed_cb (GtkWidget       *entry,
 
 	str = gtk_entry_get_text (GTK_ENTRY (window->entry_find));
 
-	is_sensitive &= !G_STR_EMPTY (str);
-	is_sensitive &= 
-		!window->last_find || 
+	is_sensitive &= !EMP_STR_EMPTY (str);
+	is_sensitive &=
+		!window->last_find ||
 		(window->last_find && strcmp (window->last_find, str) != 0);
 
 	gtk_widget_set_sensitive (window->button_find, is_sensitive);
@@ -400,7 +396,7 @@ log_window_find_populate (EmpathyLogWindow *window,
 
 	gtk_list_store_clear (store);
 
-	if (G_STR_EMPTY (search_criteria)) {
+	if (EMP_STR_EMPTY (search_criteria)) {
 		/* Just clear the search. */
 		return;
 	}
@@ -622,7 +618,7 @@ log_window_chats_populate (EmpathyLogWindow *window)
 	GtkTreeIter           iter;
 
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (window->account_chooser_chats);
-	account = empathy_account_chooser_get_account (account_chooser);
+	account = empathy_account_chooser_dup_account (account_chooser);
 
 	view = GTK_TREE_VIEW (window->treeview_chats);
 	model = gtk_tree_view_get_model (view);
