@@ -15,15 +15,15 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  *          Jonny Lamb <jonny.lamb@collabora.co.uk>
- * 
+ *
  *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
  *          Paolo Maggi
  *          Jeroen Zwartepoorte
@@ -50,7 +50,7 @@
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-idle.h>
-#include <libempathy/empathy-tp-file.h>
+#include <libempathy/empathy-ft-factory.h>
 
 #define SCHEMES "(https?|s?ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
 		"file|webcal|mailto)"
@@ -194,24 +194,25 @@ empathy_icon_name_from_account (McAccount *account)
 }
 
 const gchar *
-empathy_icon_name_for_presence (McPresence presence)
+empathy_icon_name_for_presence (TpConnectionPresenceType presence)
 {
 	switch (presence) {
-	case MC_PRESENCE_AVAILABLE:
+	case TP_CONNECTION_PRESENCE_TYPE_AVAILABLE:
 		return EMPATHY_IMAGE_AVAILABLE;
-	case MC_PRESENCE_DO_NOT_DISTURB:
+	case TP_CONNECTION_PRESENCE_TYPE_BUSY:
 		return EMPATHY_IMAGE_BUSY;
-	case MC_PRESENCE_AWAY:
+	case TP_CONNECTION_PRESENCE_TYPE_AWAY:
 		return EMPATHY_IMAGE_AWAY;
-	case MC_PRESENCE_EXTENDED_AWAY:
+	case TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY:
 		return EMPATHY_IMAGE_EXT_AWAY;
-	case MC_PRESENCE_HIDDEN:
+	case TP_CONNECTION_PRESENCE_TYPE_HIDDEN:
 		return EMPATHY_IMAGE_HIDDEN;
-	case MC_PRESENCE_OFFLINE:
-	case MC_PRESENCE_UNSET:
+	case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
+	case TP_CONNECTION_PRESENCE_TYPE_ERROR:
+	case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
 		return EMPATHY_IMAGE_OFFLINE;
-	default:
-		g_assert_not_reached ();
+	case TP_CONNECTION_PRESENCE_TYPE_UNSET:
+		return NULL;
 	}
 
 	return NULL;
@@ -220,7 +221,7 @@ empathy_icon_name_for_presence (McPresence presence)
 const gchar *
 empathy_icon_name_for_contact (EmpathyContact *contact)
 {
-	McPresence presence;
+	TpConnectionPresenceType presence;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact),
 			      EMPATHY_IMAGE_OFFLINE);
@@ -557,6 +558,29 @@ empathy_pixbuf_from_icon_name (const gchar *icon_name,
 	return empathy_pixbuf_from_icon_name_sized (icon_name, size);
 }
 
+gchar *
+empathy_filename_from_icon_name (const gchar *icon_name,
+				 GtkIconSize  icon_size)
+{
+	GtkIconTheme *icon_theme;
+	GtkIconInfo  *icon_info;
+	gint          w, h;
+	gint          size = 48;
+	gchar        *ret;
+
+	icon_theme = gtk_icon_theme_get_default ();
+
+	if (gtk_icon_size_lookup (icon_size, &w, &h)) {
+		size = (w + h) / 2;
+	}
+
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, size, 0);
+	ret = g_strdup (gtk_icon_info_get_filename (icon_info));
+	gtk_icon_info_free (icon_info);
+
+	return ret;
+}
+
 /* Stolen from GtkSourceView, hence the weird intendation. Please keep it like
  * that to make it easier to apply changes from the original code.
  */
@@ -609,7 +633,7 @@ g_utf8_strcasestr (const gchar *haystack, const gchar *needle)
 
 	if (needle_len == 0)
 	{
-		ret = (gchar *)haystack;
+		ret = (gchar *) haystack;
 		goto finally_1;
 	}
 
@@ -619,7 +643,7 @@ g_utf8_strcasestr (const gchar *haystack, const gchar *needle)
 		goto finally_1;
 	}
 
-	p = (gchar*)caseless_haystack;
+	p = (gchar *) caseless_haystack;
 	needle_len = strlen (needle);
 	i = 0;
 
@@ -715,7 +739,7 @@ forward_chars_with_skipping (GtkTextIter *iter,
 			/* being UTF8 correct sucks; this accounts for extra
 			   offsets coming from canonical decompositions of
 			   UTF8 characters (e.g. accented characters) which
-			   g_utf8_normalize() performs */
+			   g_utf8_normalize () performs */
 			gchar *normal;
 			gchar buffer[6];
 			gint buffer_len;
@@ -968,7 +992,7 @@ empathy_text_iter_forward_search (const GtkTextIter   *iter,
 		}
 	} while (gtk_text_iter_forward_line (&search));
 
-	g_strfreev ((gchar**)lines);
+	g_strfreev ((gchar **) lines);
 
 	return retval;
 }
@@ -996,7 +1020,7 @@ g_utf8_strrcasestr (const gchar *haystack, const gchar *needle)
 
 	if (needle_len == 0)
 	{
-		ret = (gchar *)haystack;
+		ret = (gchar *) haystack;
 		goto finally_1;
 	}
 
@@ -1214,7 +1238,7 @@ empathy_text_iter_backward_search (const GtkTextIter   *iter,
 		}
 	}
 
-	g_strfreev ((gchar**)lines);
+	g_strfreev ((gchar **) lines);
 
 	return retval;
 }
@@ -1240,7 +1264,7 @@ empathy_window_get_is_visible (GtkWindow *window)
 	return TRUE;
 }
 
-void 
+void
 empathy_window_iconify (GtkWindow *window, GtkStatusIcon *status_icon)
 {
 	GdkRectangle  icon_location;
@@ -1350,7 +1374,7 @@ empathy_url_show (GtkWidget *parent,
 		GtkWidget *dialog;
 
 		dialog = gtk_message_dialog_new (NULL, 0,
-						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 						 _("Unable to open URI"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 							  "%s", error->message);
@@ -1401,86 +1425,31 @@ empathy_toggle_button_set_state_quietly (GtkWidget *widget,
 	g_signal_handlers_unblock_by_func (widget, callback, user_data);
 }
 
-/* Sending files with the file chooser */
-
-static void
-file_manager_send_file_request_cb (EmpathyDispatchOperation *operation,
-				   const GError *error, gpointer user_data)
-{
-	GFile *file = (GFile *)user_data;
-	EmpathyTpFile *tp_file;
-
-	if (error != NULL) {
-		DEBUG ("Couldn't request channel: %s", error->message);
-		g_object_unref (file);
-		return;
-	}
-
-	DEBUG ("Starting to send file");
-
-	tp_file = EMPATHY_TP_FILE (
-		empathy_dispatch_operation_get_channel_wrapper (operation));
-
-	empathy_tp_file_offer (tp_file, file, NULL);
-
-	g_object_unref (file);
-}
-
 static void
 file_manager_send_file_response_cb (GtkDialog      *widget,
 				    gint            response_id,
 				    EmpathyContact *contact)
 {
+	EmpathyFTFactory *factory;
+	GFile *file;
+	gchar *uri;
+	GtkRecentManager *manager;
+
 	if (response_id == GTK_RESPONSE_OK) {
-		GSList *list;
-		GSList *l;
+		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (widget));
+		uri = g_file_get_uri (file);
 
-		list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (widget));
+		factory = empathy_ft_factory_dup_singleton ();
 
-		DEBUG ("File chooser selected files:");
+		empathy_ft_factory_new_transfer_outgoing (factory, contact,
+		                                          file);
 
-		for (l = list; l; l = l->next) {
-			gchar            *uri;
-			GFile            *gfile;
-			GFileInfo        *info;
-			GtkRecentManager *manager;
-			gchar *filename;
-			GTimeVal mtime;
-			GError *error = NULL;
+		manager = gtk_recent_manager_get_default ();
+		gtk_recent_manager_add_item (manager, uri);
 
-			uri = l->data;
-			gfile = g_file_new_for_uri (uri);
-			info = g_file_query_info (gfile,
-				G_FILE_ATTRIBUTE_STANDARD_SIZE ","
-				G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-				G_FILE_ATTRIBUTE_TIME_MODIFIED,
-				0, NULL, &error);
-
-			if (error) {
-				DEBUG ("Can't get info about the file: %s", error->message);
-				g_clear_error (&error);
-				g_object_unref (gfile);
-				continue;
-			}
-
-			DEBUG ("\t%s", uri);
-			filename = g_file_get_basename (gfile);
-			g_file_info_get_modification_time (info, &mtime);
-
-			empathy_dispatcher_send_file_to_contact (contact,
-				filename, g_file_info_get_size (info), mtime.tv_sec,
-				g_file_info_get_content_type (info),
-				file_manager_send_file_request_cb, gfile);
-
-			g_free (filename);
-			g_object_unref (info);
-
-			manager = gtk_recent_manager_get_default ();
-			gtk_recent_manager_add_item (manager, uri);
-
-			g_free (uri);
-		}
-		g_slist_free (list);
+		g_free (uri);
+		g_object_unref (factory);
+		g_object_unref (file);
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (widget));
@@ -1522,6 +1491,57 @@ empathy_send_file_with_file_chooser (EmpathyContact *contact)
 	gtk_widget_show (widget);
 }
 
+static void
+file_manager_receive_file_response_cb (GtkDialog *dialog,
+				       GtkResponseType response,
+				       EmpathyFTHandler *handler)
+{
+	EmpathyFTFactory *factory;
+	GFile *file;
+
+	if (response == GTK_RESPONSE_OK) {
+		factory = empathy_ft_factory_dup_singleton ();
+		file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (dialog));
+
+		empathy_ft_factory_set_destination_for_incoming_handler
+			(factory, handler, file);
+
+		g_object_unref (factory);
+		g_object_unref (file);
+	} else {
+		/* unref the handler, as we dismissed the file chooser,
+		 * and refused the transfer.
+		 */
+		g_object_unref (handler);
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+void
+empathy_receive_file_with_file_chooser (EmpathyFTHandler *handler)
+{
+	GtkWidget *widget;
+
+	widget = gtk_file_chooser_dialog_new (_("Select a destination"),
+					      NULL,
+					      GTK_FILE_CHOOSER_ACTION_SAVE,
+					      GTK_STOCK_CANCEL,
+					      GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_SAVE,
+					      GTK_RESPONSE_OK,
+					      NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (widget),
+		empathy_ft_handler_get_filename (handler));
+	gtk_file_chooser_set_do_overwrite_confirmation
+		(GTK_FILE_CHOOSER (widget), TRUE);
+
+	g_signal_connect (widget, "response",
+		G_CALLBACK (file_manager_receive_file_response_cb), handler);
+
+	gtk_widget_show (widget);
+}
+
 typedef struct {
 	EmpathySound sound_id;
 	const char * event_ca_id;
@@ -1624,7 +1644,7 @@ empathy_sound_play_full (GtkWidget *widget, EmpathySound sound_id,
 	       entry->event_ca_id,
 	       entry->event_ca_description);
 
-	if (ca_proplist_create(&p) < 0)
+	if (ca_proplist_create (&p) < 0)
 		goto failed;
 
 	if (ca_proplist_sets (p, CA_PROP_EVENT_ID, entry->event_ca_id) < 0)
