@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -265,7 +265,7 @@ chatroom_manager_file_parse (EmpathyChatroomManager *manager,
 
 	if (!empathy_xml_validate (doc, CHATROOMS_DTD_FILENAME)) {
 		g_warning ("Failed to validate file:'%s'", filename);
-		xmlFreeDoc(doc);
+		xmlFreeDoc (doc);
 		xmlFreeParserCtxt (ctxt);
 		return FALSE;
 	}
@@ -281,7 +281,7 @@ chatroom_manager_file_parse (EmpathyChatroomManager *manager,
 
 	DEBUG ("Parsed %d chatrooms", g_list_length (priv->chatrooms));
 
-	xmlFreeDoc(doc);
+	xmlFreeDoc (doc);
 	xmlFreeParserCtxt (ctxt);
 
 	return TRUE;
@@ -506,6 +506,27 @@ empathy_chatroom_manager_add (EmpathyChatroomManager *manager,
   return FALSE;
 }
 
+static void
+chatroom_manager_remove_link (EmpathyChatroomManager *manager,
+                              GList *l)
+{
+  EmpathyChatroomManagerPriv *priv;
+  EmpathyChatroom *chatroom;
+
+  priv = GET_PRIV (manager);
+
+  chatroom = l->data;
+
+  if (empathy_chatroom_is_favorite (chatroom))
+    reset_save_timeout (manager);
+
+  g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, chatroom);
+  g_signal_handlers_disconnect_by_func (chatroom, chatroom_changed_cb, manager);
+
+  priv->chatrooms = g_list_delete_link (priv->chatrooms, l);
+  g_object_unref (chatroom);
+}
+
 void
 empathy_chatroom_manager_remove (EmpathyChatroomManager *manager,
                                  EmpathyChatroom        *chatroom)
@@ -527,15 +548,7 @@ empathy_chatroom_manager_remove (EmpathyChatroomManager *manager,
       if (this_chatroom == chatroom ||
           empathy_chatroom_equal (chatroom, this_chatroom))
         {
-          priv->chatrooms = g_list_delete_link (priv->chatrooms, l);
-          if (empathy_chatroom_is_favorite (chatroom))
-            reset_save_timeout (manager);
-
-          g_signal_emit (manager, signals[CHATROOM_REMOVED], 0, this_chatroom);
-          g_signal_handlers_disconnect_by_func (chatroom, chatroom_changed_cb,
-              manager);
-
-          g_object_unref (this_chatroom);
+          chatroom_manager_remove_link (manager, l);
           break;
         }
     }
@@ -649,13 +662,16 @@ chatroom_manager_chat_destroyed_cb (EmpathyTpChat *chat,
         continue;
 
       empathy_chatroom_set_tp_chat (chatroom, NULL);
+
       if (!empathy_chatroom_is_favorite (chatroom))
         {
           /* Remove the chatroom from the list, unless it's in the list of
            * favourites..
            * FIXME this policy should probably not be in libempathy */
-          empathy_chatroom_manager_remove (manager, chatroom);
+          chatroom_manager_remove_link (manager, l);
         }
+
+      break;
     }
 }
 
