@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -37,7 +37,7 @@
 
 #include "empathy-contact.h"
 
-#define EMPATHY_GET_PRIV(obj,type) ((type##Priv*) ((type*)obj)->priv)
+#define EMPATHY_GET_PRIV(obj,type) ((type##Priv *) ((type *) obj)->priv)
 #define EMP_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
 
 G_BEGIN_DECLS
@@ -69,25 +69,17 @@ guint        empathy_account_hash                   (gconstpointer    key);
 gboolean     empathy_account_equal                  (gconstpointer    a,
 						    gconstpointer    b);
 MissionControl *empathy_mission_control_dup_singleton (void);
-const gchar * empathy_presence_get_default_message  (McPresence       presence);
-const gchar * empathy_presence_to_str               (McPresence       presence);
-McPresence    empathy_presence_from_str             (const gchar     *str);
+const gchar * empathy_presence_get_default_message  (TpConnectionPresenceType presence);
+const gchar * empathy_presence_to_str               (TpConnectionPresenceType presence);
+TpConnectionPresenceType empathy_presence_from_str  (const gchar     *str);
 gchar *       empathy_file_lookup                   (const gchar     *filename,
 						     const gchar     *subdir);
-
-typedef gboolean (*EmpathyRunUntilReadyFunc)        (GObject         *object,
-						     gpointer         user_data);
-void          empathy_run_until_ready               (gpointer         object);
-void          empathy_run_until_ready_full          (gpointer         object,
-						     const gchar     *signal,
-						     EmpathyRunUntilReadyFunc  func,
-						     gpointer         user_data,
-						     GMainLoop      **loop);
-McAccount *  empathy_channel_get_account            (TpChannel       *channel);
 gboolean     empathy_proxy_equal                    (gconstpointer    a,
 						     gconstpointer    b);
 guint        empathy_proxy_hash                     (gconstpointer    key);
 gboolean     empathy_check_available_state          (void);
+gint        empathy_uint_compare                    (gconstpointer a,
+						     gconstpointer b);
 
 G_END_DECLS
 
