@@ -76,7 +76,6 @@ typedef struct {
 	/* Menu items. */
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
-	GtkAction   *menu_conv_contact;
 	GtkAction   *menu_conv_favorite;
 
 	GtkAction   *menu_edit_cut;
@@ -162,21 +161,39 @@ chat_window_close_clicked_cb (GtkAction   *action,
 }
 
 static void
-chat_window_close_button_style_set_cb (GtkWidget *button,
+chat_tab_style_set_cb (GtkWidget *hbox,
 				       GtkStyle  *previous_style,
 				       gpointer   user_data)
 {
-	gint h, w;
+	GtkWidget *button;
+	int char_width, h, w;
+	PangoContext *context;
+	PangoFontMetrics *metrics;
+
+	button = g_object_get_data (G_OBJECT (user_data),
+		"chat-window-tab-close-button");
+	context = gtk_widget_get_pango_context (hbox);
+
+	metrics = pango_context_get_metrics (context, hbox->style->font_desc,
+		pango_context_get_language (context));
+	char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	pango_font_metrics_unref (metrics);
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
 					   GTK_ICON_SIZE_MENU, &w, &h);
 
+	/* Request at least about 12 chars width plus at least space for the status
+	 * image and the close button */
+	gtk_widget_set_size_request (hbox,
+		12 * PANGO_PIXELS (char_width) + 2 * w, -1);
+
 	gtk_widget_set_size_request (button, w, h);
 }
 
 static GtkWidget *
 chat_window_create_label (EmpathyChatWindow *window,
-			  EmpathyChat       *chat)
+			  EmpathyChat       *chat,
+			  gboolean           is_tab_label)
 {
 	EmpathyChatWindowPriv *priv;
 	GtkWidget            *hbox;
@@ -198,7 +215,8 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE);
 
 	name_label = gtk_label_new (NULL);
-	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
+	if (is_tab_label)
+		gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
 
 	attr_list = pango_attr_list_new ();
 	attr = pango_attr_scale_new (1/1.2);
@@ -210,7 +228,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
-	g_object_set_data (G_OBJECT (chat), "chat-window-tab-label", name_label);
+	g_object_set_data (G_OBJECT (chat),
+		is_tab_label ? "chat-window-tab-label" : "chat-window-menu-label",
+		name_label);
 
 	status_image = gtk_image_new ();
 
@@ -220,42 +240,47 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), status_image, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), name_label, TRUE, TRUE, 0);
 
-	g_object_set_data (G_OBJECT (chat), "chat-window-tab-image", status_image);
-	g_object_set_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget", event_box);
-
-	close_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
-	g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
-
-	/* We don't want focus/keynav for the button to avoid clutter, and
-	 * Ctrl-W works anyway.
-	 */
-	GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
-	GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_DEFAULT);
-
-	/* Set the name to make the special rc style match. */
-	gtk_widget_set_name (close_button, "empathy-close-button");
-
-	close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-
-	gtk_container_add (GTK_CONTAINER (close_button), close_image);
+	g_object_set_data (G_OBJECT (chat),
+		is_tab_label ? "chat-window-tab-image" : "chat-window-menu-image",
+		status_image);
+	g_object_set_data (G_OBJECT (chat),
+		is_tab_label ? "chat-window-tab-tooltip-widget" : "chat-window-menu-tooltip-widget",
+		event_box);
 
 	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
-	gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
-	/* React to theme changes and also used to setup the initial size
-	 * correctly.
-	 */
-	g_signal_connect (close_button,
-			  "style-set",
-			  G_CALLBACK (chat_window_close_button_style_set_cb),
-			  chat);
+	if (is_tab_label) {
+		close_button = gtk_button_new ();
+		gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
 
-	g_signal_connect (close_button,
-			  "clicked",
-			  G_CALLBACK (chat_window_close_clicked_cb),
-			  chat);
+		/* We don't want focus/keynav for the button to avoid clutter, and
+		 * Ctrl-W works anyway.
+		 */
+		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
+		GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_DEFAULT);
+
+		/* Set the name to make the special rc style match. */
+		gtk_widget_set_name (close_button, "empathy-close-button");
+
+		close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+
+		gtk_container_add (GTK_CONTAINER (close_button), close_image);
+
+		gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
+
+		g_signal_connect (close_button,
+				  "clicked",
+				  G_CALLBACK (chat_window_close_clicked_cb),
+				  chat);
+
+		/* React to theme changes and also setup the size correctly.  */
+		g_signal_connect (hbox,
+				  "style-set",
+				  G_CALLBACK (chat_tab_style_set_cb),
+				  chat);
+	}
 
 	gtk_widget_show_all (hbox);
 
@@ -279,6 +304,8 @@ chat_window_update (EmpathyChatWindow *window)
 	gboolean               avatar_in_icon;
 	GtkWidget             *chat;
 	GtkWidget             *chat_close_button;
+	GtkWidget             *submenu;
+	GtkWidget             *menu;
 
 	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -299,6 +326,13 @@ chat_window_update (EmpathyChatWindow *window)
 	gtk_action_set_sensitive (priv->menu_tabs_right, !last_page);
 	gtk_action_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
 
+	/* Update Contact menu */
+	menu = gtk_ui_manager_get_widget (priv->ui_manager,
+		"/chats_menubar/menu_contact");
+	submenu = empathy_chat_get_contact_menu (priv->current_chat);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
+	gtk_widget_show (menu);
+
 	/* Update window title */
 	if (n_chats == 1) {
 		gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
@@ -347,6 +381,23 @@ chat_window_update (EmpathyChatWindow *window)
 	}
 }
 
+static void
+append_markup_printf (GString    *string,
+		      const char *format,
+		      ...)
+{
+	gchar *tmp;
+	va_list args;
+
+	va_start (args, format);
+
+	tmp = g_markup_vprintf_escaped (format, args);
+	g_string_append (string, tmp);
+	g_free (tmp);
+
+	va_end (args);
+}
+
 static void
 chat_window_update_chat_tab (EmpathyChat *chat)
 {
@@ -354,8 +405,10 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	EmpathyChatWindowPriv *priv;
 	EmpathyContact        *remote_contact;
 	const gchar           *name;
+	const gchar           *id;
 	McAccount             *account;
 	const gchar           *subject;
+	const gchar           *status = NULL;
 	GtkWidget             *widget;
 	GString               *tooltip;
 	gchar                 *markup;
@@ -390,44 +443,49 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	}
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
 	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_MENU);
+	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-image");
+	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_MENU);
 
 	/* Update tab tooltip */
 	tooltip = g_string_new (NULL);
 
 	if (remote_contact) {
-		markup = g_markup_printf_escaped ("<b>%s</b><small> (%s)</small>\n%s",
-						  empathy_contact_get_id (remote_contact),
-						  mc_account_get_display_name (account),
-						  empathy_contact_get_status (remote_contact));
-		g_string_append (tooltip, markup);
-		g_free (markup);
+		id = empathy_contact_get_id (remote_contact);
+		status = empathy_contact_get_presence_message (remote_contact);
+	} else {
+		id = name;
 	}
-	else {
-		markup = g_markup_printf_escaped ("<b>%s</b><small>  (%s)</small>", name,
-						  mc_account_get_display_name (account));
-		g_string_append (tooltip, markup);
-		g_free (markup);
+
+	append_markup_printf (tooltip,
+			      "<b>%s</b><small> (%s)</small>",
+			      id,
+			      mc_account_get_display_name (account));
+
+	if (!EMP_STR_EMPTY (status)) {
+		append_markup_printf (tooltip, "\n<i>%s</i>", status);
 	}
 
 	if (subject) {
-		markup = g_markup_printf_escaped ("\n<b>%s</b> %s", _("Topic:"), subject);
-		g_string_append (tooltip, markup);
-		g_free (markup);
+		append_markup_printf (tooltip, "\n<b>%s</b> %s",
+				      _("Topic:"), subject);
 	}
+
 	if (g_list_find (priv->chats_composing, chat)) {
-		markup = g_markup_printf_escaped ("\n%s", _("Typing a message."));
-		g_string_append (tooltip, markup);
-		g_free (markup);
+		append_markup_printf (tooltip, "\n%s", _("Typing a message."));
 	}
 
 	markup = g_string_free (tooltip, FALSE);
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget");
 	gtk_widget_set_tooltip_markup (widget, markup);
+	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-tooltip-widget");
+	gtk_widget_set_tooltip_markup (widget, markup);
 	g_free (markup);
 
-	/* Update tab label */
+	/* Update tab and menu label */
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
 	gtk_label_set_text (GTK_LABEL (widget), name);
+	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-label");
+	gtk_label_set_text (GTK_LABEL (widget), name);
 
 	/* Update the window if it's the current chat */
 	if (priv->current_chat == chat) {
@@ -488,20 +546,8 @@ chat_window_conv_activate_cb (GtkAction         *action,
 			      EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	GtkWidget             *menu = NULL;
-	GtkWidget             *submenu = NULL;
 	gboolean               is_room;
 
-	/* Contact submenu */
-	submenu = empathy_chat_get_contact_menu (priv->current_chat);
-	if (submenu) {
-		menu = gtk_ui_manager_get_widget (priv->ui_manager,
-			"/chats_menubar/menu_conv/menu_conv_contact");
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
-		gtk_widget_show (submenu);
-	}
-	gtk_action_set_visible (priv->menu_conv_contact, submenu != NULL);
-
 	/* Favorite room menu */
 	is_room = empathy_chat_is_room (priv->current_chat);
 	if (is_room) {
@@ -930,24 +976,15 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	/* - if we're the sender, we play the sound if it's specified in the
 	 *   preferences and we're not away.
 	 * - if we receive a message, we play the sound if it's specified in the
-	 *   prefereces and the window does not have focus on the chat receiving
+	 *   preferences and the window does not have focus on the chat receiving
 	 *   the message.
 	 */
 
 	sender = empathy_message_get_sender (message);
 
-	if (empathy_contact_is_user (sender) != FALSE) {
+	if (empathy_contact_is_user (sender)) {
 		empathy_sound_play (GTK_WIDGET (priv->dialog),
 				    EMPATHY_SOUND_MESSAGE_OUTGOING);
-	} else {
-		if ((!has_focus || priv->current_chat != chat)) {
-			empathy_sound_play (GTK_WIDGET (priv->dialog),
-					    EMPATHY_SOUND_MESSAGE_INCOMING);
-		}
-	}
-
-	if (!has_focus) {
-		chat_window_show_or_update_notification (window, message, chat);
 	}
 
 	if (has_focus && priv->current_chat == chat) {
@@ -966,8 +1003,13 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 		needs_urgency = TRUE;
 	}
 
-	if (needs_urgency && !has_focus) {
-		chat_window_set_urgency_hint (window, TRUE);
+	if (needs_urgency) {
+		if (!has_focus)
+			chat_window_set_urgency_hint (window, TRUE);
+
+		empathy_sound_play (GTK_WIDGET (priv->dialog),
+		    EMPATHY_SOUND_MESSAGE_INCOMING);
+		chat_window_show_or_update_notification (window, message, chat);
 	}
 
 	if (!g_list_find (priv->chats_new_msg, chat)) {
@@ -1326,7 +1368,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "chat_vbox", &chat_vbox,
 				       "ui_manager", &priv->ui_manager,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
-				       "menu_conv_contact", &priv->menu_conv_contact,
 				       "menu_conv_favorite", &priv->menu_conv_favorite,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
@@ -1362,7 +1403,9 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	priv->chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 
 	priv->notebook = gtk_notebook_new ();
- 	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow");
+	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow");
+	gtk_notebook_set_scrollable (GTK_NOTEBOOK (priv->notebook), TRUE);
+	gtk_notebook_popup_enable (GTK_NOTEBOOK (priv->notebook));
 	gtk_box_pack_start (GTK_BOX (chat_vbox), priv->notebook, TRUE, TRUE, 0);
 	gtk_widget_show (priv->notebook);
 
@@ -1508,6 +1551,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 {
 	EmpathyChatWindowPriv *priv;
 	GtkWidget             *label;
+	GtkWidget             *popup_label;
 	GtkWidget             *child;
 	gint                   x, y, w, h;
 
@@ -1539,7 +1583,8 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	}
 
 	child = GTK_WIDGET (chat);
-	label = chat_window_create_label (window, chat);
+	label = chat_window_create_label (window, chat, TRUE);
+	popup_label = chat_window_create_label (window, chat, FALSE);
 	gtk_widget_show (child);
 
 	g_signal_connect (chat, "notify::name",
@@ -1553,7 +1598,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 			  NULL);
 	chat_window_chat_notify_cb (chat);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), child, label);
+	gtk_notebook_append_page_menu (GTK_NOTEBOOK (priv->notebook), child, label, popup_label);
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (priv->notebook), child,
