@@ -31,6 +31,10 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
+#if HAVE_LIBCHAMPLAIN
+#include <clutter-gtk/gtk-clutter-embed.h>
+#endif
+
 #include <libebook/e-book.h>
 #include <libnotify/notify.h>
 
@@ -46,11 +50,13 @@
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-log-manager.h>
+#include <libempathy/empathy-ft-factory.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-call.h>
 
 #include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
+#include <libempathy-gtk/empathy-location-manager.h>
 
 #include "empathy-accounts-dialog.h"
 #include "empathy-main-window.h"
@@ -124,15 +130,17 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
 		factory = empathy_call_factory_get ();
 		empathy_call_factory_claim_channel (factory, operation);
 	} else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER) {
-		EmpathyFTManager *ft_manager;
-		EmpathyTpFile    *tp_file;
+		EmpathyFTFactory *factory;
 
-		ft_manager = empathy_ft_manager_dup_singleton ();
-		tp_file = EMPATHY_TP_FILE (
-			empathy_dispatch_operation_get_channel_wrapper (operation));
-		empathy_ft_manager_add_tp_file (ft_manager, tp_file);
-		empathy_dispatch_operation_claim (operation);
-		g_object_unref (ft_manager);
+		factory = empathy_ft_factory_dup_singleton ();
+
+		/* if the operation is not incoming, don't claim it,
+		 * as it might have been triggered by another client, and
+		 * we are observing it.
+		 */
+		if (empathy_dispatch_operation_is_incoming (operation)) {
+			empathy_ft_factory_claim_channel (factory, operation);
+		}
 	}
 }
 
@@ -403,26 +411,57 @@ show_version_cb (const char *option_name,
 	return FALSE;
 }
 
+static void
+new_incoming_transfer_cb (EmpathyFTFactory *factory,
+			  EmpathyFTHandler *handler,
+                          GError *error,
+			  gpointer user_data)
+{
+	if (error) {
+		empathy_ft_manager_display_error (handler, error);
+	} else {
+		empathy_receive_file_with_file_chooser (handler);
+	}
+}
+
+static void
+new_ft_handler_cb (EmpathyFTFactory *factory,
+		   EmpathyFTHandler *handler,
+                   GError *error,
+		   gpointer user_data)
+{
+	if (error) {
+		empathy_ft_manager_display_error (handler, error);
+	} else {
+		empathy_ft_manager_add_handler (handler);
+	}
+
+	g_object_unref (handler);
+}
+
 static void
 new_call_handler_cb (EmpathyCallFactory *factory, EmpathyCallHandler *handler,
 	gboolean outgoing, gpointer user_data)
 {
-		EmpathyCallWindow *window;
+	EmpathyCallWindow *window;
 
-		window = empathy_call_window_new (handler);
-		gtk_widget_show (GTK_WIDGET (window));
+	window = empathy_call_window_new (handler);
+	gtk_widget_show (GTK_WIDGET (window));
 }
 
 int
 main (int argc, char *argv[])
 {
 	guint32            startup_timestamp;
+#if HAVE_GEOCLUE
+	EmpathyLocationManager *location_manager = NULL;
+#endif
 	EmpathyStatusIcon *icon;
 	EmpathyDispatcher *dispatcher;
 	EmpathyLogManager *log_manager;
 	EmpathyChatroomManager *chatroom_manager;
-	EmpathyFTManager  *ft_manager;
 	EmpathyCallFactory *call_factory;
+	EmpathyFTFactory  *ft_factory;
 	GtkWidget         *window;
 	MissionControl    *mc;
 	EmpathyIdle       *idle;
@@ -466,6 +505,10 @@ main (int argc, char *argv[])
 
 	gst_init (&argc, &argv);
 
+#if HAVE_LIBCHAMPLAIN
+	gtk_clutter_init (&argc, &argv);
+#endif
+
 	gtk_window_set_default_icon_name ("empathy");
 	textdomain (GETTEXT_PACKAGE);
 
@@ -537,7 +580,8 @@ main (int argc, char *argv[])
 			       EMPATHY_PREFS_AUTOCONNECT,
 			       &autoconnect);
 	if (autoconnect && ! no_connect &&
-	    empathy_idle_get_state (idle) <= MC_PRESENCE_OFFLINE) {
+		tp_connection_presence_type_cmp_availability (empathy_idle_get_state
+			(idle), TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0) {
 		empathy_idle_set_state (idle, MC_PRESENCE_AVAILABLE);
 	}
 	
@@ -565,17 +609,26 @@ main (int argc, char *argv[])
 	chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 	empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
 
-	ft_manager = empathy_ft_manager_dup_singleton ();
-
 	notify_init (_(PACKAGE_NAME));
 	/* Create the call factory */
 	call_factory = empathy_call_factory_initialise ();
 	g_signal_connect (G_OBJECT (call_factory), "new-call-handler",
 		G_CALLBACK (new_call_handler_cb), NULL);
+	/* Create the FT factory */
+	ft_factory = empathy_ft_factory_dup_singleton ();
+	g_signal_connect (ft_factory, "new-ft-handler",
+		G_CALLBACK (new_ft_handler_cb), NULL);
+	g_signal_connect (ft_factory, "new-incoming-transfer",
+		G_CALLBACK (new_incoming_transfer_cb), NULL);
+
+	/* Location mananger */
+#if HAVE_GEOCLUE
+	location_manager = empathy_location_manager_dup_singleton ();
+#endif
 
 	gtk_main ();
 
-	empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
+	empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_OFFLINE);
 
 	g_object_unref (mc);
 	g_object_unref (idle);
@@ -583,7 +636,10 @@ main (int argc, char *argv[])
 	g_object_unref (log_manager);
 	g_object_unref (dispatcher);
 	g_object_unref (chatroom_manager);
-	g_object_unref (ft_manager);
+#if HAVE_GEOCLUE
+	g_object_unref (location_manager);
+#endif
+	g_object_unref (ft_factory);
 
 	notify_uninit ();
 
